/*
 * Copyright (c) 1997.  Netscape Communications Corporation.  All
 * rights reserved.
 * 
 * Search the directory for all people whose surname (last name) is
 * "Jensen".  Since the "sn" attribute is a caseignorestring (cis), case
 * is not significant when searching.
 *
 * You can control whether to block on receiving all results or on one
 * at a time by setting the batchSize.
 *
 */

import netscape.ldap.*;
import java.util.*;

public class Search {
    public static void main( String[] args )
	{
		LDAPConnection ld = null;
		int status = -1;
		try {
			ld = new LDAPConnection();
			/* Connect to server */
			String MY_HOST = "osprey4.npac.syr.edu";
			int MY_PORT = 2323;
			ld.connect( MY_HOST, MY_PORT );

			/* search for all entries with surname of Jensen */
			String MY_FILTER = "(&(filename=*)(o=*))";//objectclass=*";
			String MY_SEARCHBASE = "o=WebFlow,c=US";
			String atts[] = {"filename","hostname"};
			receiveSiteAndMachines(ld,MY_SEARCHBASE);
			//receiveAttributes(ld,atts,MY_SEARCHBASE);
			

			LDAPSearchConstraints cons = ld.getSearchConstraints();
			/* Setting the batchSize to one will cause the result
			   enumeration below to block on one result at a time,
			   allowing us to update a list or do other things as
			   results come in. */
			/* We could set it to 0 if we just wanted to get all
			   results and were willing to block until then */
			cons.setBatchSize( 1 );
			LDAPSearchResults res = ld.search( MY_SEARCHBASE,
												LDAPConnection.SCOPE_SUB,
												MY_FILTER,
							   atts,//null,
												false,
												cons );

			/* Loop on results until finished */
			while ( res.hasMoreElements() ) {

				/* Next directory entry */
				LDAPEntry findEntry = (LDAPEntry)res.nextElement();
				System.out.println( findEntry.getDN() );

				/* Get the attributes of the entry */
				LDAPAttributeSet findAttrs = findEntry.getAttributeSet();
				Enumeration enumAttrs = findAttrs.getAttributes();
				System.out.println( "\tAttributes: " );
				/* Loop on attributes */
				while ( enumAttrs.hasMoreElements() ) {
					LDAPAttribute anAttr =
						(LDAPAttribute)enumAttrs.nextElement();
					String attrName = anAttr.getName();
					System.out.println( "\t\t" + attrName );
					/* Loop on values for this attribute */
					Enumeration enumVals = anAttr.getStringValues();
					while ( enumVals.hasMoreElements() ) {
						String aVal = ( String )enumVals.nextElement();
						System.out.println( "\t\t\t" + aVal );
					}
				}
			}
			status = 0;
		}
		catch( LDAPException e ) {
			System.out.println( "Error: " + e.toString() );
		}

		/* Done, so disconnect */
		if ( (ld != null) && ld.isConnected() ) {
			try {
			    ld.disconnect();
			} catch ( LDAPException e ) {
				System.out.println( "Error: " + e.toString() );
			}
		}
		System.exit(status);
	}


public static void receiveSiteAndMachines (LDAPConnection ld  , String searchBase)
  {
       
    String []atts = {"filename","machine" ,"organization"};
    String searchFilter = "(&(filename=*)(machine=*)(organization=*))";
    Vector pairs = new Vector();

    /* Read all attributes */
    try {
    
      LDAPSearchConstraints cons = ld.getSearchConstraints();
      cons.setBatchSize( 1 );
      LDAPSearchResults res = ld.search( searchBase, LDAPConnection.SCOPE_SUB,
		       searchFilter, atts,  false,     cons );

      /* Loop on results until finished */
      while ( res.hasMoreElements() ) {
	LDAPEntry findEntry = (LDAPEntry)res.nextElement();
	String DNName = findEntry.getDN() ;
	System.out.println( "DN: "+DNName);

	LDAPAttributeSet findAttrs = findEntry.getAttributeSet();
	Enumeration enumAttrs = findAttrs.getAttributes();

        String onePair[]  = new String[2];
	while ( enumAttrs.hasMoreElements() ) {
	  LDAPAttribute anAttr = (LDAPAttribute)enumAttrs.nextElement();
	  String attrName = anAttr.getName();
	  Enumeration enumVals = anAttr.getStringValues();
	  System.out.println("AttributeName:"+attrName+":");
	  
	  while ( enumVals.hasMoreElements() ) {
	    String aVal = ( String )enumVals.nextElement();
	    if (attrName.equals("machine"))
	      onePair[0] = aVal ;
	   	    
	    else if (attrName.equals("organization"))
	      onePair[1] = aVal ; 
	  }
	  	    
	}
	System.out.println("consider pair :"+onePair[0]+" "+onePair[1]);
	String concatPair = onePair[0]+":"+onePair[1];
	
	if (pairs.indexOf(concatPair) < 0){
	  System.out.println("Add to PaIR");
	  pairs.addElement(concatPair);
	}
	
      }
      
    } catch( LDAPException e ) 
      {
	System.out.println( "SearchFull:Error: " + e.toString() );
      }


    for(int k=0;k<pairs.size();k++)
      {
        String onepair = (String)pairs.elementAt(k);
	System.out.println("pair :"+onepair);
      }
        
  }


public static void receiveAttributes (LDAPConnection ld ,String[] attributeNames,
				 String searchBase)
  {
    String searchFilter="(&";
 
    int numOfAttributes = attributeNames.length;
    String[] atts = new String[numOfAttributes];
    for(int k=0;k<numOfAttributes;k++){
      atts[k] = attributeNames[k];
      searchFilter += "("+atts[k]+"=*)";
    }
    searchFilter += ")";
    System.out.println("searchFilter :"+searchFilter);
        
    
   Vector tempVals  = new Vector();
   Hashtable tempAtts = new Hashtable();
   Hashtable result = new Hashtable();

    /* Read all attributes */
    try {

      LDAPSearchConstraints cons = ld.getSearchConstraints();
      cons.setBatchSize( 1 );
      LDAPSearchResults res = ld.search( searchBase, LDAPConnection.SCOPE_SUB,
		       searchFilter, atts,  false,     cons );

      /* Loop on results until finished */
      while ( res.hasMoreElements() ) {
	LDAPEntry findEntry = (LDAPEntry)res.nextElement();
	String DNName = findEntry.getDN() ;
	System.out.println( DNName);

	LDAPAttributeSet findAttrs = findEntry.getAttributeSet();
	Enumeration enumAttrs = findAttrs.getAttributes();
	tempAtts.clear();
	while ( enumAttrs.hasMoreElements() ) {
	  LDAPAttribute anAttr = (LDAPAttribute)enumAttrs.nextElement();
	  String attrName = anAttr.getName();
	  Enumeration enumVals = anAttr.getStringValues();
	  tempVals.removeAllElements();
	  
	  while ( enumVals.hasMoreElements() ) {
	    String aVal = ( String )enumVals.nextElement();
	    tempVals.addElement(aVal);
	  }
	  tempAtts.put(attrName,tempVals);
	}
	if (!result.contains(tempAtts))
	  result.put(DNName,tempAtts);
      }
    } catch( LDAPException e ) 
      {
	System.out.println( "SearchFull:Error: " + e.toString() );
      }


    for(Enumeration list = result.elements(),keys=result.keys(); 
	list.hasMoreElements();)
      {
	String dnname = (String)keys.nextElement();
	System.out.println("DNName :"+dnname);
	
	Hashtable table = (Hashtable)list.nextElement();
	for(Enumeration inlist = table.elements(),inkeys=table.keys(); 
	    inlist.hasMoreElements();){
	  String attname = (String)inkeys.nextElement();
	  System.out.println("attname:"+attname);

	  Vector alist = (Vector)inlist.nextElement();
	  for(int k=0;k<alist.size();k++){
	    String val = (String)alist.elementAt(k);
	    System.out.println("val:"+val);
	  }
	  
	}
      }
    
    
    
  }
}
